/*    Copyright (C) 2010  Tomasz Bursztyka
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <log.h>

static log_level_t master_level = INFO_LVL;
static FILE *log_file = NULL;

extern in_plugin_handlers_t *iph;

log_msg_f log_msg = log_msg_df;

void init_log_level(log_level_t master_lvl)
{
	master_level = master_lvl;
}

int init_logger(char *log_filename)
{
	if (log_filename != NULL)
	{
		log_file = fopen(log_filename, "a+");

		if (log_file == NULL)
		{
			log_file = stdout;
			log_msg(ERROR_LVL, "Was not able to initiate logging with file %s. Using standard output instead.\n", log_file);
			return -1;
		}
	} 
	else
	{
		log_file = stdout;
		log_msg(WARNING_LVL, "Logging will use standard output\n");
	}

	log_msg(INFO_LVL, "Logging initiated\n");

	return 0;
}


void log_msg_df(log_level_t lvl, const char *format, ...)
{
	va_list args;
	time_t yet;
	char *time_line = NULL;

	if (log_file == NULL)
	{
		log_file = stdout;
#ifdef DEBUG
		master_level = DEBUG_LVL;
#endif
	}

	if (master_level >= lvl)
	{
		va_start(args, format);

		yet = time(NULL);
		time_line = asctime((const struct tm *) localtime(&yet));
		time_line[strlen(time_line)-1] = '\0';

		fprintf(log_file, "%s:", time_line);

		switch (lvl)
		{
		case FATAL_LVL:
			fprintf(log_file, " [FATAL] ");
			break;
		case ERROR_LVL:
			fprintf(log_file, " [ERROR] ");
			break;
		case WARNING_LVL:
			fprintf(log_file, " [WARNING] ");
			break;
		case INFO_LVL:
			fprintf(log_file, " [INFO] ");
			break;
#ifdef DEBUG
		case DEBUG_LVL:
			fprintf(log_file, " [DEBUG] ");
			break;
#endif
		case MAX_LOG_LVL:
		default:
			break;
		}
		vfprintf(log_file, format, args);

		fflush(log_file);
	}
}

void log_set_redirection(log_msg_f log_redirection)
{
	log_msg = log_redirection;
	iph->log_msg_h = log_redirection;
}

void log_unset_redirection(void)
{
	log_msg = log_msg_df;
	iph->log_msg_h = log_msg_df;
}
