/*    Copyright (C) 2010  Tomasz Bursztyka
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <vpzone_internal.h>
#include <vpzone.h>
#include <configuration.h>
#include <command.h>
#include <iface.h>
#include <io.h>
#include <plugin.h>
#include <log.h>
#include <sighdl.h>

const handler_t default_ih[HDL_MAX_HANDLERS] =
{
	manage_packet_df,
	route_packet_df,
	NULL,
	read_tun_df,
	write_tun_df,
	remove_mnode_df,
	remove_mserver_df,
	remove_mdevice_df
};

handler_t ih[HDL_MAX_HANDLERS];
callback_t *icb[CB_MAX_CALLBACKS];

mdevice_t *mdevice_g = NULL;

int main(int argc, char *argv[])
{
	const char *all_params = CONFIG_PARAMS;
	int long_index;
	int option;
	char *config_file_path = NULL;

	config_file_path = DEFAULT_CONFIGURATION_FILE;

	setup_sighdl();

	while ((option = getopt_long(argc, argv, all_params,
								 vpzone_config_opts, &long_index)) != -1)
	{
		switch(option)
		{
		case 'd':
			if (fork() != 0)
				exit(EXIT_SUCCESS);
			else
				setsid();
			break;
		case 'f':
			config_file_path = optarg;
			break;
		case 'h':
			fprintf(stdout, VPZONE_HELP_MSG);
			exit(EXIT_SUCCESS);
			break;
		case '?':
			fprintf(stdout, VPZONE_HELP_MSG);
			exit(EXIT_FAILURE);
			break;
		default:
			break;
		}
	}

	memcpy(ih, default_ih, sizeof(handler_t) * HDL_MAX_HANDLERS);
	memset(icb, 0, sizeof(callback_t*) * CB_MAX_CALLBACKS);

	if (init_commands() < 0)
	{
		log_msg(FATAL_LVL, "Could not initialize command set\n");
		exit(EXIT_FAILURE);
	}

	mdevice_g = init_mdevice(config_file_path);

	setup_plugin_manager();

	setup_io(mdevice_g);

	parse_configuration(config_file_path, NULL);

	if (init_mdevice_hash(mdevice_g) != 0)
		return EXIT_FAILURE;

	DEBUG_MSG("Polling...\n");

	while (true)
		poll_io(mdevice_g);

	return 0;
}
