/*    Copyright (C) 2010  Tomasz Bursztyka
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef __HASH_H__
#define __HASH_H__

#include <stdbool.h>
#include <stdio.h>
#include <stdint.h>
#include <stdlib.h>
#include <string.h>

struct _hentry_s
{
	void *key;
	void *value;
	struct _hentry_s *next;
};
typedef struct _hentry_s hentry_t;

typedef int (*keycmp_f) (void *, void*);
typedef uint32_t (*hash_f) (void *);

struct _hiterator_s
{
	hentry_t *cur_entry;
	hentry_t *parent;
	uint32_t index;
};

typedef struct _hiterator_s hiterator_t;

struct _hashmap_s
{
	uint32_t nb_entries;
	uint32_t length;
	hentry_t **entries;
	keycmp_f key_compare;
	hash_f hash_compute;
	hiterator_t iterator;
};
typedef struct _hashmap_s hashmap_t;

#define HASH_INDEX(h, l) (uint32_t) (h % l)

void *h_create(uint32_t size, hash_f hash_compute_function, keycmp_f key_cmp_function);

int h_insert(void *hm, void *key, void *value);

void *h_search(void *hm, void *key);

void *h_remove(void *hm, void *key, bool free_key);

int h_destroy(void *hm);

uint32_t h_count(void *hm);

int h_iter_begin(void *hm);

void h_iter_end(void *hm);

bool h_iter_next(void *hm);

void *h_iter_getkey(void *hm);

void *h_iter_getvalue(void *hm);

#endif
