/*    Copyright (C) 2010  Tomasz Bursztyka
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef __LOG_H__
#define __LOG_H__

#include <stdio.h>
#include <stdarg.h>
#include <time.h>
#include <string.h>
#include <errno.h>

#include <vpzone_internal.h>
#include <vpzone.h>

extern log_msg_f log_msg;

/*Macros*/

#define DEFAULT_LOG_FILE "/var/log/vpzone.log"

#ifdef DEBUG

#define ERR(fn)													\
	log_msg(ERROR_LVL,											\
			"%s:%d %s - code %d - %s\n",						\
			__FILE__, __LINE__, fn, errno, strerror(errno));

#define DEBUG_MSG(...) log_msg(DEBUG_LVL, __VA_ARGS__)

#else /* ifndef DEBUG*/

#define ERR(fn)							 \
	log_msg(ERROR_LVL,					 \
			"%s - code %d - %s\n",		 \
			fn, errno, strerror(errno));

#define DEBUG_MSG(...) {}

#endif /*DEBUG*/

void init_log_level(log_level_t master_lvl);

int init_logger(char *log_filename);

void log_msg_df(log_level_t lvl, const char *format, ...);

void log_set_redirection(log_msg_f log_redirection);

void log_unset_redirection(void);

#endif
