/*    Copyright (C) 2010  Tomasz Bursztyka
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <gnutls_cmd.h>

extern in_plugin_handlers_t *ph;

extern gnutls_certificate_credentials_t gnutls_cred;

extern gnutls_dh_params_t gnutls_dh;

int cafile_cmd(int argc, char **argv)
{
	if (argc != 2)
	{
		log_msg(WARNING_LVL, "cafile: needs one argument (and one only)\n");
		return CMD_ERROR;
	}

	if (gnutls_certificate_set_x509_trust_file(gnutls_cred, argv[1], GNUTLS_X509_FMT_PEM) < 0)
		return CMD_ERROR;

	return CMD_SUCCESS;
}

int certnkey_files_cmd(int argc, char **argv)
{
	if (argc != 3)
	{
		log_msg(WARNING_LVL, "certnkey-files: needs 2 arguments\n");
		return CMD_ERROR;
	}

	if (gnutls_certificate_set_x509_key_file(gnutls_cred, argv[1], argv[2], GNUTLS_X509_FMT_PEM) != GNUTLS_E_SUCCESS)
		return CMD_ERROR;

	return CMD_SUCCESS;
}

int dhfile_cmd(int argc, char **argv)
{
	struct stat dh_stat;
	char *dh_content = NULL;
	FILE *dh_fd = NULL;
	gnutls_datum_t dparams;

	if (argc != 2)
	{
		log_msg(WARNING_LVL, "dhfile: needs one argument (and one only)\n");
		return CMD_ERROR;
	}

	if(stat(argv[1], &dh_stat) < 0)
	{
		ERR("dhfile_cmd()");
		return CMD_ERROR;
	}

	if (dh_stat.st_size > DH_FILE_MAX_SIZE)
		return CMD_ERROR;

	if((dh_content = (char *) malloc(sizeof(char)*(dh_stat.st_size+1))) == NULL)
	{
		ERR("dhfile_cmd()");
		return CMD_ERROR;
	}

	memset(dh_content, 0, sizeof(char)*(dh_stat.st_size+1));

	if((dh_fd = fopen(argv[1], "r")) == NULL)
	{
		ERR("dhfile_cmd()");
		free(dh_content);
		return CMD_ERROR;
	}

	if(fread(dh_content, dh_stat.st_size, 1, dh_fd) != 1)
	{
		if(feof(dh_fd))
			log_msg(WARNING_LVL, "Seems we are at DH file end... trying to continue\n");
		else
			log_msg(WARNING_LVL, "Seems we are not at DH file end... trying to continue\n");
	}

	fclose(dh_fd);

	dparams.data = (unsigned char *) dh_content;
	dparams.size = dh_stat.st_size;

	gnutls_dh_params_init (&gnutls_dh);

	if (gnutls_dh_params_import_pkcs3(gnutls_dh, &dparams, GNUTLS_X509_FMT_PEM) != GNUTLS_E_SUCCESS)
	{
		free(dh_content);
		return CMD_ERROR;
	}

	gnutls_certificate_set_dh_params(gnutls_cred, gnutls_dh);

	free(dh_content);

	return CMD_SUCCESS;
}

int crlfile_cmd(int argc, char **argv)
{
	if (argc != 2)
	{
		log_msg(WARNING_LVL, "crlfile: needs one argument (and one only)\n");
		return CMD_ERROR;
	}

	if (gnutls_certificate_set_x509_crl_file(gnutls_cred, argv[1], GNUTLS_X509_FMT_PEM) < 0)
		return CMD_ERROR;

	return CMD_SUCCESS;
}
